@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: CGIResponseWriter.h 2508 2007-07-06 22:47:18Z svn $
+ * $Id: CGIResponseWriter.h 2540 2007-07-21 15:21:47Z svn $
  *****************************************************************************/
 
 #ifndef CGI_RESPONSE_WRITER_H
@@ -30,9 +30,13 @@
 
 #include "Environment.h"
 
-#include <iostream>
-
 #include "apr_file_io.h"
+#if APR_HAVE_SYS_UIO_H
+#define APR_WANT_IOVEC
+#include "apr_want.h"
+#else
+#include <iostream>
+#endif
 
 #include "ResponseWriter.h"
 #include "CGIResponse.h"
@@ -45,34 +49,51 @@ class CGIResponseWriter: public ResponseWriter
 {
 public:
     CGIResponseWriter(CGIResponse::Handle *r=NULL)
+        : iovec_count_(0)
     {
+        pool_ = r->pool;
+    };
+    ~CGIResponseWriter()
+    {
+        flush();
+    };
+    void write(const char *str, apr_size_t length)
+    {
+#if APR_HAVE_SYS_UIO_H
+        iovec_[iovec_count_].iov_base   = const_cast<char *>(str);
+        iovec_[iovec_count_].iov_len    = length;
+        iovec_count_++;
 
-    };
-    void write(const char *str, apr_size_t length) const
-    {
+        if (UNLIKELY(iovec_count_ == IO_VECTOR_SIZE)) {
+            flush();
+        }
+#else
         std::cout.write(str, static_cast<std::streamsize>(length));
-#ifdef DEBUG_TemplateExecutor
-        std::cout.flush();
 #endif
     };
-    void write(const char *str) const
+    void write(const char *str)
     {
-        std::cout << str;
-#ifdef DEBUG_TemplateExecutor
-        std::cout.flush();
-#endif
+        write(str, strlen(str));
     };
-    void write(int i) const
+    void write(int i)
     {
-        std::cout << i;
-#ifdef DEBUG_TemplateExecutor
-        std::cout.flush();
-#endif
+        write(apr_itoa(pool_, i));
     };
     void finish() const
     {
         std::cout.flush();
     };
+    void flush()
+    {
+        if (iovec_count_ == 0) {
+            return;
+        }
+
+#if APR_HAVE_SYS_UIO_H
+        writev(STDOUT_FILENO, iovec_, iovec_count_);
+#endif
+        iovec_count_ = 0;
+    };
     static bool is_sendfile_enabled(CGIResponse::Handle *r){
         return false;
     };
@@ -88,6 +109,12 @@ public:
 
         return OK;
     };
+
+private:
+    static const apr_size_t IO_VECTOR_SIZE = RES_IO_VECTOR_SIZE;
+
+    struct iovec iovec_[IO_VECTOR_SIZE];
+    apr_size_t iovec_count_;
 };
 
 #endif
