@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: ReadWriteLocker.h 2513 2007-07-09 13:05:22Z svn $
+ * $Id: ReadWriteLocker.h 2540 2007-07-21 15:21:47Z svn $
  *****************************************************************************/
 
 #ifndef READ_WRITE_LOCKER_H
@@ -30,8 +30,11 @@
 
 #include "Environment.h"
 
+#define DEBUG_LOCK
+
 #ifdef DEBUG_LOCK
 #include <iostream>
+#include <iomanip>
 #endif
 
 #ifdef HAVE_UNISTD_H
@@ -71,7 +74,8 @@ protected:
                                                   AGE_MASK |
                                                   READ_COUNT_MASK;
 
-    static const apr_uint32_t WRITE_LOCKED      = 0xFFFF & READ_COUNT_MASK;
+    static const apr_uint32_t INVALID_AGE       = 0xFF;
+    static const apr_uint32_t WRITE_LOCKED      = READ_COUNT_MASK;
     static const apr_uint32_t PREFER_WRITE      = 1 << PREFER_WRITE_SHIFT;
     static const apr_uint32_t PREFER_NONE       = 0 << PREFER_WRITE_SHIFT;
     static const apr_uint32_t WAIT_WRITE        = 1 << WAIT_WRITE_SHIFT;
@@ -80,30 +84,28 @@ protected:
     static const apr_uint32_t READ_COUNT_UINIT  = 1 << READ_COUNT_SHIFT;
 
     static const apr_uint32_t TIMEOUT_CHECK_MASK= 0x1F;
-    static const apr_uint32_t TIMEOUT_SEC       = 180;
+    static const apr_uint32_t TIMEOUT_SEC       = 300;
 
     /**
      * コンストラクタです．
-     *
-     * @param[out] lock ロック変数
      */
-    ReadWriteLocker(apr_atomic_t *lock)
-     : lock_(lock),
-       start_age_(0),
-       start_time_(0),
-       status_(0)
+    ReadWriteLocker()
+     : start_age_(INVALID_AGE)
     {
 
     };
 
-    ReadWriteLocker(const ReadWriteLocker& locker)
-      : lock_(locker.lock_)
+    /**
+     * ロック後の初期化を行います．
+     *
+     * @param[in] lock ロック変数
+     * @param[in] status ロックの状態
+     */
+    void init(apr_atomic_t *lock, apr_uint32_t status)
     {
-
+        lock_ = lock;
+        status_ = status;
     };
-
-    ReadWriteLocker();
-
     // 注意: ReadWriteLocker 型のポインタ経由で使用されることはないので，
     // デストラクタは virtual にしない．
 
@@ -155,14 +157,14 @@ protected:
     {
 #ifdef DEBUG_LOCK
         std::cerr << "try_count[" << label << "] = " << try_count << std::endl;
-        std::cerr << "state: " << hex << curr_status << " -> ";
-        std::cerr << hex << new_status << std::endl;
+        std::cerr << "state: " << std::hex << curr_status << " -> ";
+        std::cerr << std::hex << new_status << std::endl;
         std::cerr.flush();
 #endif
     };
 
     /** 読み出し中のスレッドの数へのポインタ */
-    apr_atomic_t * const lock_;
+    apr_atomic_t *lock_;
     apr_uint32_t start_age_;
     apr_time_t start_time_;
     apr_uint32_t status_;
